#ifndef oxygenexceptionmodel_h
#define oxygenexceptionmodel_h
//////////////////////////////////////////////////////////////////////////////
// oxygenexceptionmodel.h
// -------------------
//
// Copyright (c) 2009 Hugo Pereira Da Costa <hugo.pereira@free.fr>
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to
// deal in the Software without restriction, including without limitation the
// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
// sell copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
// IN THE SOFTWARE.
//////////////////////////////////////////////////////////////////////////////

#include "oxygenlistmodel.h"
#include "oxygensettings.h"
#include "oxygen.h"

namespace Oxygen
{

    //* qlistview for object counters
    class ExceptionModel: public ListModel<InternalSettingsPtr>
    {

        public:

        //* number of columns
        enum { nColumns = 3 };

        //* column type enumeration
        enum ColumnType {
            ColumnEnabled,
            ColumnType,
            ColumnRegExp
        };


        //*@name methods reimplemented from base class
        //@{

        //* return data for a given index
        virtual QVariant data(const QModelIndex &index, int role) const;

        //* header data
        virtual QVariant headerData(int section, Qt::Orientation orientation, int role = Qt::DisplayRole) const;

        //* number of columns for a given index
        virtual int columnCount(const QModelIndex& ) const
        { return nColumns; }

        //@}

        protected:

        //* sort
        virtual void privateSort( int, Qt::SortOrder )
        {}

        private:

        //* column titles
        static const QString m_columnTitles[ nColumns ];

    };

}
#endif
